c
      program grnest
c
c     This program estimates the load distribution using
c     the ground displacement.  Green's function of Farrell (1972) is used.
c
      parameter (maxload=500,maxpara=500,maxsit=1500)
      implicit real*4 (a-h,o-z)
      real*8 dxyz(3)
      dimension cent(2,maxload),xlat(maxsit),xlon(maxsit),vert(maxsit)
     1         ,displ(3),area(maxload)
     2         ,rlat(2),rlon(2),a(maxpara,maxsit),neig(maxload)
     3         ,b(maxpara,maxpara),binv(maxpara,maxpara)
     4         ,x(maxpara),z(maxpara),aty(maxpara)
      character lbuf(37)*1,lsite(maxsit)*4,lstn*4
     1         ,lvert*8,lminusz*14
      data rlat/34.5,41.25/
      data rlon/136.5, 142.5/
      data arclen/110.95/  ! length of 1 degree at 35N in km
      pi=4.0*atan(1.0)
      dp=pi/1.8e2
c
      xwidth=1.0/6.0
      ywidth=1.0/4.0
c
c-----parameters to be tuned
c
      continuity=100.0    ! adjacent blocks is allowed to be different by 100 cm
      yerr=5.0            ! measurement error of displacement is 5.0 mm
      rigidity=30.0       ! rigidity in GPa
c
c-----read load center from file
c
      open(1,file='input/blocks_e.lod')
        iload=1
        read(1,510)
  510   format(/)
 1111   read(1,520)clat,(lbuf(k),k=1,37)
  520   format(f5.2,1x,37a1)
        if(clat.eq.0.0) goto 2222
        do k=1,37
          if(lbuf(k).ne.'.') then
            clon=136.5+(k-1)*xwidth
            cent(1,iload)=clat
            cent(2,iload)=clon
            neig(iload)=0
            tate=1.e3*arclen*ywidth
            yoko=1.e3*arclen*cos(clat*dp)*xwidth
            area(iload)=tate*yoko    ! area of this block in m^2
            iload=iload+1
          endif
        enddo
        goto 1111
 2222   mload=iload-1
      close(1)
c     write(6,*) mload
c
c-----rest normal matrix
c
      do iload=1,mload
        do kload=1,mload
          b(iload,kload)=0.0
        enddo
        aty(iload)=0.0
      enddo
c
c-----continuity constraint
c
      do iload=2,mload
        ie1=nint((cent(2,iload)-136.5)/xwidth) + 1
        in1=nint((cent(1,iload)-34.5)/ywidth) + 1
        do kload=1,iload-1
          ie2=nint((cent(2,kload)-136.5)/xwidth) + 1
          in2=nint((cent(1,kload)-34.5)/ywidth) + 1
          id_e=iabs(ie2-ie1)
          id_n=iabs(in2-in1)
          if(id_e+id_n.eq.1) then
            b(kload,iload)=b(kload,iload)-1.0/continuity**2
            b(iload,kload)=b(iload,kload)-1.0/continuity**2
            b(iload,iload)=b(iload,iload)+1.0/continuity**2
            b(kload,kload)=b(kload,kload)+1.0/continuity**2
            neig(iload)=neig(iload)+1
            neig(kload)=neig(kload)+1
          endif
        enddo
      enddo
c
c-----Tikhonov constraint for coastal stations (continuity to zero value at ocean)
c
      do iload=1,mload
c       write(6,*) neig(iload)
        if(neig(iload).ne.4) then   ! a certain facet of the block faces the ocean
          b(iload,iload)=b(iload,iload) + 1.0/continuity**2
        endif
      enddo
c
c-----reading GNSS station coordinates 
c
      isit=1
 5555 read(5,200,err=6666,end=6666)xloni,xlati,lvert
  200 format(2f10.4,a8)
      read(lvert,*,err=5555) verti
      if(xloni.gt.rlon(2).or.xloni.lt.rlon(1)) goto 5555
      if(xlati.gt.rlat(2).or.xlati.lt.rlat(1)) goto 5555
      xlon(isit)=xloni
      xlat(isit)=xlati
      vert(isit)=10.0*verti    ! convert unit from cm (in data file) to mm
      isit=isit+1
      goto 5555
 6666 nsit=isit-1
c
c-----calculate displacement site by site
c 
      do isit=1,nsit
        do iload=1,mload
          call dispfrl(xlat(isit),xlon(isit),cent(1,iload)
     1                      ,cent(2,iload),rigidity,displ)
          a(iload,isit)=displ(3)      ! displ(3) is vertical movement for 100 Pa (1 hPa)
        enddo
c
        do k1=1,mload     ! adding to the normal matrix
          do k2=1,mload
            b(k1,k2)=b(k1,k2)+a(k1,isit)*a(k2,isit)/yerr**2
          enddo
          aty(k1)=aty(k1)+a(k1,isit)*vert(isit)/yerr**2
        enddo
c
      enddo
c
c-----least-squares calculation
c
      call lsq(b,aty,z,maxpara,mload,x,binv)
c
c     output parameters for GMT 
c
      volume=0.0
      do iload=1,mload
        write(lminusz,'("> -Z",f10.2)') x(iload)   ! output load in unit of cm (hPa)
        call rem_space(lminusz)
        write(6,'(a14)') lminusz
        write(6,*) cent(2,iload)-0.5*xwidth,cent(1,iload)-0.5*ywidth
        write(6,*) cent(2,iload)-0.5*xwidth,cent(1,iload)+0.5*ywidth
        write(6,*) cent(2,iload)+0.5*xwidth,cent(1,iload)+0.5*ywidth
        write(6,*) cent(2,iload)+0.5*xwidth,cent(1,iload)-0.5*ywidth
        write(6,*) cent(2,iload)-0.5*xwidth,cent(1,iload)-0.5*ywidth
        volume=volume + area(iload)*0.01*x(iload)   ! unit is m^3
      enddo
c
c------getting residuals
c
      open(1,file='data/calcv') 
      rms=0.0
      rms2=0.0
      do isit=1,nsit
        ycalc=0.0
        do iload=1,mload
          ycalc=ycalc+a(iload,isit)*x(iload)
        enddo
        write(1,*) xlon(isit),xlat(isit),0.1*ycalc   ! back to cm in output
        res=vert(isit)-ycalc
        rms=rms+res**2
        rms2=rms2+vert(isit)**2
      enddo
      close(1)
      rms=sqrt(rms/real(nsit))
      rms2=sqrt(rms2/real(nsit))
c
c  enable these write sentences to output statistical results
c
c     write(6,130) mload,nsit,rms,rms2,1.e-9*volume
c    1            ,1.e-9*volume_pos
c 130 format('# load:',i5,5x,'# site:',i5,5x,/'rms (mm):',f5.2,
c    2   ' (',f5.2,')',5x,'total water:',f6.1,' km^3')
c
c-----terminate
c
      stop
      end
c
c===================================================================
c
      subroutine rem_space(lminusz)
c
      character lminusz*14
c
      ip1=5
      do ip=ip1+1,14
        if(lminusz(ip:ip).ne.' ') then
          ip2=ip
          goto 1000
        endif
      enddo
c
 1000 continue
      nchar=ip2-ip1
c
      do ip=ip2,14
        lminusz(ip-nchar:ip-nchar)=lminusz(ip:ip)
      enddo
      do ip=14-nchar+1,14
        lminusz(ip:ip)=' '
      enddo
c
      return
      end
c
c===================================================================
c
      subroutine dispfrl(xlat,xlon,slat,slon,rigidity,displ)
c
      implicit real*4 (a-h,o-z)
      parameter (mp=10)
c
      dimension disp(3),displ(3)
      data rearth/6.37101e+6/     ! Earth's mean radius (m)
      gacc=9.8
      xwidth=1.0/6.0
      ywidth=1.0/4.0
c 
      pi=4.0*atan(1.0)
      dp=pi/1.8e2
      wdens=1.e3
      rgty=1.e9*rigidity
      sigma=3.0*rgty
      eta=2.0*rgty
      sovere=3.0/2.0
c
      fsr=cos(slat*dp)
      xsep=fsr*xwidth*dp/real(mp)
      ysep=    ywidth*dp/real(mp)
      wheight=1.e-2
      area=xsep*rearth*ysep*rearth
      force=gacc*wheight*area*wdens
c
c     here we assume snow load as deep as 1 cm in equivalent water,
c     which corresponds to about 100 Pa (N/m2)
c
      disp(1)=0.0
      disp(2)=0.0
      disp(3)=0.0
c
      do iy=1, mp
        sslat = slat - 0.5*ywidth + (real(iy)-0.5)*ywidth/real(mp)
        yy=(xlat-sslat)*dp*rearth
c
        do ix=1,mp
          sslon = slon - 0.5*xwidth + (real(ix)-0.5)*xwidth/real(mp)
          xx=fsr*(xlon-sslon)*dp*rearth
          dist=sqrt(yy**2 + xx**2)
          bumbo1=4.0*pi*rgty*dist
          bumbo2=4.0*pi*eta*dist
          vdisp=force*sovere/bumbo1
          hdisp=force/bumbo2
c
          disp_u=-vdisp          ! add minus because displacement is up positive
          disp_n=-hdisp*yy/dist
          disp_e=-hdisp*xx/dist
c
          disp(1)=disp(1)+disp_n
          disp(2)=disp(2)+disp_e
          disp(3)=disp(3)+disp_u
        enddo
c
      enddo
c
      do k=1,3
        displ(k)=1.e3*disp(k)     ! convert from meter to millimeter
      enddo
c
      return
      end
c
c===================================================================
c
      subroutine lsq(b,aty,z,mdim,ndim,x,binv)
c*
      implicit real*4 (a-h,o-z)
c
c declarations
c -------------
c
      dimension b(mdim,mdim),binv(mdim,mdim),aty(mdim),x(mdim)
     1      ,z(mdim)
c
c cholesky separation of normal matrix
c ------------------------------------
c
      call rrsep(b,mdim,ndim,binv)
c
c solve the first equation
c ------------------------
c
      call rsolv2(binv,aty,mdim,ndim,z)
c
c solve the second equation
c -------------------------
c
      call rsolv1(binv,z,mdim,ndim,x)
c
c get inverse matrix of r matrix
c ------------------------------
c
      call rinvs(binv,z,mdim,ndim,b)
c
c square the inverse of r matrix
c ------------------------------
c
      call rsqar(b,mdim,ndim,binv)
c
c return to the main
c ------------------
c
      return
      end
c
c=============================================================
c
      subroutine rrsep(cov,mdim,ndim,r)
c
      implicit real*4 (a-h,o-z)
c
      dimension cov(mdim,mdim),r(mdim,mdim)
c
      r(1,1)=sqrt(cov(1,1))
      do k=2,ndim
        r(1,k)=cov(1,k)/r(1,1)
      enddo
c
      do k2=2,ndim
c
        do k3=1,k2-1
          r(k2,k3)=0.0
        enddo
c
        do k3=k2,ndim
c
          r(k2,k3)=cov(k2,k3)
          do k1=1,k2-1
            r(k2,k3)=r(k2,k3)-r(k1,k2)*r(k1,k3)
          enddo
c
          if(k3.eq.k2) then
            r(k2,k3)=sqrt(r(k2,k3))
          else
            r(k2,k3)=r(k2,k3)/r(k2,k2)
          endif
c
        enddo
c
      enddo
c
      return
      end
c
c=============================================================
c
      subroutine rsolv2(r,scr1,mdim,ndim,scr2)
c
      implicit real*4 (a-h,o-z)
c
      dimension r(mdim,mdim),scr1(mdim),scr2(mdim)
c
      scr2(1)=scr1(1)/r(1,1)
      do k2=2,ndim
        scr2(k2)=scr1(k2)
        do k1=1,k2-1
          scr2(k2)=scr2(k2)-r(k1,k2)*scr2(k1)
        enddo
        scr2(k2)=scr2(k2)/r(k2,k2)
      enddo
c
      return
      end
c
c=============================================================
c
      subroutine rsolv1(r,scr1,mdim,ndim,scr2)
c
      implicit real*4 (a-h,o-z)
c
      dimension r(mdim,mdim),scr1(mdim),scr2(mdim)
c
      scr2(ndim)=scr1(ndim)/r(ndim,ndim)
      do k2=ndim-1,1,-1
        scr2(k2)=scr1(k2)
        do k1=k2+1,ndim
          scr2(k2)=scr2(k2)-r(k2,k1)*scr2(k1)
        enddo
        scr2(k2)=scr2(k2)/r(k2,k2)
      enddo 
c
      return
      end
c
c=============================================================
c
      subroutine rinvs(r,scr,mdim,ndim,rinv)
c
      implicit real*4 (a-h,o-z)
c
      dimension r(mdim,mdim),rinv(mdim,mdim),scr(mdim)
c
      do i=1,ndim
        do k=1,ndim
          scr(k)=0.0
        enddo
        scr(i)=1.0
c
        call rsolv1(r,scr,mdim,ndim,rinv(1,i))
c
      enddo
c
      return
      end
c
c=============================================================
c
      subroutine rsqar(r,mdim,ndim,wt)
c
      implicit real*4 (a-h,o-z)
c
      dimension r(mdim,mdim),wt(mdim,mdim)
c
      do k2=1,ndim
        do k1=k2,ndim
          wt(k1,k2)=0.0
          do k=k1,ndim
            wt(k1,k2)=wt(k1,k2)+r(k1,k)*r(k2,k)
          enddo
          if(k1.ne.k2) wt(k2,k1)=wt(k1,k2)
        enddo
      enddo
c
      return
      end

